Program Queuesim;         { Single queue simulation program }
USES Crt,Queue_AS;        { All times in minutes }

Const MinInt   = 1;       { Minimum arrival interval }
      MaxInt   = 15;      { Maximum arrival interval }
      MinServe = 0;       { Minimum service duration }
      MaxServe = 12;      { Maximum service duration }
      MaxTime  = 10000;   { Experiment duration }
      Trace = false;      { Whether tracing or not }

Var WQueue  : QueueP;     { Waiting queue }
    Time    : Real;       { Model time }
    NextArr : Real;       { Next arrival }
    NextEOS : Real;       { Next end of service }
    TotalWT : Real;       { Total waiting time }
    TotalST : Real;       { Total time spent in system }
    CustNum : Longint;    { Customer number }
    TotalNum: Longint;    { Number of customers served }
    Idle    : Boolean;    { Server status }
    MaxQueue: Integer;    { Maximum queue length }
    Cust    : ElemType;   { Customer }
    SCust   : ElemType;   { Served customer }
    Finish  : Boolean;
    c       : char;

Begin
   { Model initialisation: }
   Create(WQueue);          { Creating an empty queue }
   Time := 0;
   TotalWT := 0; TotalST := 0; CustNum := 0;
   TotalNum := 0; MaxQueue := 0;
   Idle := true; Finish := false; c := ' ';
   ClrScr;
   { Scheduling the first arrival: }
   NextArr := MinInt + Random*(MaxInt-MinInt);   { 1st arrival }
   Repeat
      If (NextArr<NextEOS) or Idle then begin    { Arrival}
         Time := NextArr;
         NextArr := Time + MinInt + Random*(MaxInt-MinInt);
	 { Next arrival scheduled }
         Inc(CustNum);
         Cust.Number := CustNum;
         Cust.ArrTime := Time;
         If Trace then begin
            TextColor(LightBlue);
            Write('Customer ',CustNum,' came at ',Time:5:2);
	    GoToXY(40,WhereY);
	    Writeln(' Queue: ',WQueue^.Size,'          (Quit/Continue)');
           c := UpCase(ReadKey);
         end;
         If not Idle then                  { Queueing }
	  If not Full(WQueue) then begin { Testing queue }
	     Enqueue(WQueue,Cust);
             If WQueue^.Size>MaxQueue then MaxQueue:=WQueue^.Size;
           end
          else begin
             Write ('Queue full, press Enter');
             Readln; Halt;
          end
         Else begin                { Starting service }
            SCust := Cust;
            SCust.StartTime := Time;
            Idle := false;
            NextEOS := Time + MinServe + Random*(MaxServe-MinServe);
         end
       end{ Arrival }

      Else begin                     { End of service }
         Time := NextEOS;
         TextColor(Yellow);
         If Trace then begin
           Write('Customer ',SCust.Number,' served at ',Time:5:2);
	        GoToXY(40,WhereY);
           Writeln(' Queue: ',WQueue^.Size,'          (Quit/Continue)');
           c := UpCase(ReadKey);
         end;
         Inc(TotalNum);                  { Statistics }
         TotalWT := TotalWt + SCust.StartTime - SCust.ArrTime;
         TotalST := TotalST + Time - SCust.ArrTime;
         If not Empty(WQueue) then begin   { Testing queue }
            Serve(WQueue,SCust);        { Starting service }
            SCust.StartTime := Time;
            Idle := false;
            NextEOS := Time + MinServe + Random*(MaxServe-MinServe);
          end
         Else Idle := true;
      end { End of service };

      Finish := (c='Q') or (Time>=MaxTime);
   Until Finish;

{ Evaluation: }
   TextColor(LightGreen);
   Writeln;
   Writeln('EVALUATION AT TIME',Time:10:2); Writeln;
	Writeln('Total number of customers served  =', TotalNum:8);
   If TotalNum=0 then TotalNum:=1;
	Writeln('Average waiting time in the queue =', TotalWT/TotalNum:8:3);
	Writeln('Average time spent in the system  =', TotalST/TotalNum:8:3);
   Writeln('Maximum queue length              =',MaxQueue:8);
   Readln;
End.

